==> glx.ui-1.8.21.tgz: package/components/studentSearch/studentSearch.ts <==
import { AfterViewInit, Component, EventEmitter, Input, NgModule, OnInit, Output, Renderer2, ViewChild } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { ActivatedRoute, Router, RouterModule } from '@angular/router';

import { DataTable, DataTableModule } from 'primeng/components/datatable/datatable';
import { DropdownModule } from 'primeng/components/dropdown/dropdown';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';
import { SharedModule } from 'primeng/components/common/shared';

import { ProfileViewerModule } from '../profileViewer/profileViewer';
import { LoadingBannerModule } from '../loadingBanner/loadingBanner';
import { SliderModule } from '../slider/slider';

import { fadeInOutAnimation } from '../kernel/animations/fade.animation';

import { GlxSharedModule } from '../kernel/shared.module';

import 'rxjs/add/operator/switchMap';

class Filter {
    fullname: string;
    edu_specialization: string;
    department: string;
    edu_qualification: string;
    edu_course: string;
    edu_year: string;
    edu_form: string;
    global: string;

    constructor() {

    }
}

@Component({
    selector: 'glx-student-search',
    styleUrls: ['./studentSearch.scss'],
    templateUrl: './studentSearch.html',
    animations: [fadeInOutAnimation]
})


export class StudentSearch implements OnInit, AfterViewInit {
    @Input() data: any = {};
    @Input() catalog: any = {};
    @Input() profile: any = {};
    @Input() filterShow: boolean = false;
    @Input() loading: boolean = false;
    @Input() slider: boolean = false;
    @Input() defaultImage: string = '';
    @Input() filterDelay: number = 1000;
    @Input() filter: Filter = new Filter();
    @Input() globalFilterPlaceholder: string = 'Поиск...';
    @Output() onFilter: EventEmitter<any> = new EventEmitter();
    @Output() onViewProfile: EventEmitter<any> = new EventEmitter();
    @Output() onViewManager: EventEmitter<any> = new EventEmitter();
    @Output() exportAction: EventEmitter<any> = new EventEmitter();

    @Input() filterField: any;
    @Input() filterValue: any;
    @Input() imagePrefix: string;

    @ViewChild('dt') datatable: DataTable;
    @ViewChild('gb') gb: any;
    @ViewChild('filterDepartment') filterDepartment: any;

    showSide: boolean = false;  // показать профиль
    profile_id: number;

    filterTimeout: any;
    firstFilter: boolean = true;

    constructor(private router: Router, private route: ActivatedRoute, private renderer: Renderer2) {
    }

    ngOnInit() {
        this.checkCatalog();
        this.route.paramMap.subscribe(
            params => {
                if (!this.slider) {
                    this.filterField = params.get('field');
                    this.filterValue = params.get('value');
                    if (!this.firstFilter) {
                        this.checkPreFilter();
                        this.datatable._filter();
                        if (this.showSide) {
                            this.toggleSide();
                        }
                    }
                }
            }
        );
    }

    getPhoto(photo) {
        if (!photo) {
            return null;
        }
        return this.imagePrefix + photo;
    }

    checkPreFilter() {
        if (this.route.snapshot.paramMap.get('field') && this.route.snapshot.paramMap.get('value') && this.firstFilter) {
            this.filterField = this.route.snapshot.paramMap.get('field');
            this.filterValue = this.route.snapshot.paramMap.get('value');
        }

        if (this.filterField && this.filterValue) {
            this.filter[this.filterField] = this.filterValue;
            if (this.filterValue === parseInt(this.filterValue, 10)) {
                this.datatable.filters[this.filterField] = {
                    value: parseInt(this.filterValue, 10),
                    matchMode: 'any'
                };
            } else {
                this.datatable.filters[this.filterField] = {
                    value: this.filterValue,
                    matchMode: 'any'
                };
            }
        }
    }

    ifDisabled(field) {
        return !!(this.filterField && this.filterValue && this.filterField === field);
    }

    ngAfterViewInit() {
        this.renderer.listen(
            this.filterDepartment.filterViewChild.nativeElement,
            'keyup',
            (event: KeyboardEvent) => {
                const key = event.which || event.keyCode;

                if (key === 13 || key === 40) {
                    this.filterDepartment.applyFocus();
                    /**
                     * из кода primeng/components/dropdown/dropdown.ts строка 419
                     * обработка нажатия клавиши вниз, чтобы после фильтрации выбрать первый отфильтрованнный элемент
                     */
                    const selectedItemIndex = this.filterDepartment.selectedOption ? this.filterDepartment.findOptionIndex(
                        this.filterDepartment.selectedOption.value,
                        this.filterDepartment.optionsToDisplay
                    ) : -1;
                    if (selectedItemIndex !== -1) {
                        const nextItemIndex = selectedItemIndex + 1;
                        if (nextItemIndex !== (this.filterDepartment.optionsToDisplay.length)) {
                            this.filterDepartment.selectedOption = this.filterDepartment.optionsToDisplay[nextItemIndex];
                            this.filterDepartment.selectedOptionUpdated = true;
                            this.filterDepartment.selectItem(event, this.filterDepartment.selectedOption);
                        }
                    } else if (this.filterDepartment.optionsToDisplay) {
                        this.filterDepartment.selectedOption = this.filterDepartment.optionsToDisplay[0];
                        this.filterDepartment.selectedOptionUpdated = true;
                        this.filterDepartment.selectItem(event, this.filterDepartment.selectedOption);
                    }
                }
            });
    }

    /**
     * Скрыть/показать профиль пользователя
     */
    toggleSide() {
        this.showSide = !this.showSide;
    }

    clearFilter() {
        if (this.filterField || this.filterValue) {
            this.filterField = null;
            this.filterValue = null;

            if (!this.slider) {
                this.router.navigate(['/search/student']);
            }
        }

        this.filter = new Filter();
        if (this.datatable.globalFilter && this.datatable.globalFilter.value) {
            this.datatable.globalFilter.value = '';
        }
        this.datatable.reset();
        this.datatable.sortField = 'fullname';
        this.datatable._filter();
    }

    /**
     * Скрыть / показать фильтры
     */
    toggleFilter() {
        this.filterShow = !this.filterShow;
    }

    /**
     * Применение глобального фильтра
     * @param gbt
     */
    globalFilterApply(gbt) {
        if (this.filterTimeout) {
            clearTimeout(this.filterTimeout);
        }
        if (gbt && gbt.value && gbt.value.length > 2) {
            this.filterTimeout = setTimeout(() => {
                this.datatable.globalFilter = gbt;
                this.datatable._filter();
                this.filterTimeout = null;
            }, this.filterDelay);
        }
        if (gbt && gbt.value === '') {
            this.datatable.globalFilter.value = '';
            this.datatable._filter();
        }
    }

    setProfileHeader() {
        if (this.profile && this.profile.role && this.profile.role.length > 0) {
            return this.profile.user.fullname;
        }
        return '';
    }

    /**
     * Установка и применение фильтра
     * @param name
     * @param value
     */
    setFilter(name, value) {
        switch (name) {
            case 'fullname':
                if (this.filterTimeout) {
                    clearTimeout(this.filterTimeout);
                }
                if (value && value.length > 2) {
                    this.filterTimeout = setTimeout(() => {
                        this.datatable.filter(value, name, 'any');
                        this.filterTimeout = null;
                    }, this.filterDelay);
                }
                if (value === '') {
                    this.datatable.filter(value, name, 'any');
                }
                break;
            default:
                this.datatable.filter(value, name, 'any');
        }

    }

    loadItemsLazy(event: any) {
        if (this.firstFilter) {
            this.checkPreFilter();
            this.firstFilter = false;
        }

        this.onFilter.emit(event);
    }

    /**
     * Устанавилвает фильтр из контекста (при клике на значение в таблице)
     * @param name string Название поля для фильтрации
     * @param value string Значение поля
     */
    setContextFilter(name, value) {
        this.filter = new Filter();
        this.datatable.filters = {};
        this.datatable.filteredValue = null;
        this.datatable.sortField = 'fullname';
        this.filter[name] = value;
        this.setFilter(name, value);
    }

    /**
     * заполнение каталогов если они пустые
     */
    checkCatalog() {
        if (!this.catalog.years || this.catalog.years.length === 0) {
            this.initCatalogYears(7);
        }
        if (!this.catalog.edu_forms || this.catalog.edu_forms.length === 0) {
            this.initCatalogForms();
        }
        if (!this.catalog.edu_courses || this.catalog.edu_courses.length === 0) {
            this.initCatalogCourses(6);
        }
    }

    initCatalogYears(number) {
        this.catalog.years = [];
        const year = new Date().getFullYear();
        this.catalog.years.push(
            {label: '(любой)', value: ''}
        );
        for (let i = 0, y = year; i < number; i++, y--) {
            this.catalog.years.push(
                {label: y, value: y}
            );
        }
    }

    initCatalogForms() {
        this.catalog.edu_forms = [];
        this.catalog.edu_forms.push(
            {label: '(любая)', value: ''},
            {label: 'очное обучение', value: 'full-time'},
            {label: 'заочное обучение', value: 'distance'},
            {label: 'очно-заочное обучение', value: 'part-time'},
        );
    }

    initCatalogCourses(number) {
        this.catalog.edu_courses = [];
        this.catalog.edu_courses.push(
            {label: '(любой)', value: ''}
        );
        for (let i = 1; i <= number; i++) {
            this.catalog.edu_courses.push(
                {label: i, value: i}
            );
        }
    }

    showProfile(item ?) {
        if (item) {
            this.profile_id = item.id;
        }

        if (this.profile_id > 0) {
            this.onViewProfile.emit(this.profile_id);
            this.toggleSide();
        } else {
            console.error('Ошибка: Не получен ID профиля');
        }
    }

    onDblRowclick() {
        this.showProfile();
    }

    onRowClick(event) {
        this.profile_id = event.data.id;
    }

    /**
     * Установка подсказки в фильтре подразделений
     * @param item
     * @returns {any}
     */
    getLabel(item) {
        if (item && item.tree && item.tree.length > 0) {
            let deps = '';
            for (let i = 0; i < item.tree.length; i++) {
                deps += '<div class="dep" style="padding-left: ' + (i * 8) + 'px">' + item.tree[i] + '</div>';
            }
            return `<div class="tooltip-custom-content">` + deps + `</div>`;
        } else {
            return '';
        }

    }

    managerViewHandler(e) {
        this.onViewManager.emit({id: e.id});
    }

    export() {
        this.exportAction.emit({filter: this.datatable.createLazyLoadMetadata()});
    }

}

@NgModule({
    imports: [
        CommonModule,
        FormsModule,
        RouterModule,
        SharedModule,
        GlxSharedModule,
        DataTableModule,
        DropdownModule,
        TooltipModule,
        ProfileViewerModule,
        LoadingBannerModule,
        SliderModule
    ],
    exports: [StudentSearch],
    declarations: [StudentSearch]
})

export class StudentSearchModule {
}

==> glx.ui-1.8.37.tgz: package/components/studentSearch/studentSearch.ts <==
import { AfterViewInit, Component, EventEmitter, Input, NgModule, OnInit, Output, Renderer2, ViewChild } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { ActivatedRoute, Router, RouterModule } from '@angular/router';

import { DataTable, DataTableModule } from 'primeng/components/datatable/datatable';
import { DropdownModule } from 'primeng/components/dropdown/dropdown';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';
import { SharedModule } from 'primeng/components/common/shared';

import { ProfileViewerModule } from '../profileViewer/profileViewer';
import { LoadingBannerModule } from '../loadingBanner/loadingBanner';
import { SliderModule } from '../slider/slider';

import { fadeInOutAnimation } from '../kernel/animations/fade.animation';

import { GlxSharedModule } from '../kernel/shared.module';

import 'rxjs/add/operator/switchMap';

class Filter {
    fullname: string;
    edu_specialization: string;
    department: string;
    edu_qualification: string;
    edu_course: string;
    edu_year: string;
    edu_form: string;
    global: string;

    constructor() {

    }
}

@Component({
    selector: 'glx-student-search',
    styleUrls: ['./studentSearch.scss'],
    templateUrl: './studentSearch.html',
    animations: [fadeInOutAnimation]
})


export class StudentSearch implements OnInit, AfterViewInit {
    @Input() data: any = {};
    @Input() catalog: any = {};
    @Input() profile: any = {};
    @Input() filterShow: boolean = false;
    @Input() loading: boolean = false;
    @Input() slider: boolean = false;
    @Input() defaultImage: string = '';
    @Input() filterDelay: number = 1000;
    @Input() filter: Filter = new Filter();
    @Input() globalFilterPlaceholder: string = 'Поиск...';
    @Output() onFilter: EventEmitter<any> = new EventEmitter();
    @Output() onViewProfile: EventEmitter<any> = new EventEmitter();
    @Output() onViewManager: EventEmitter<any> = new EventEmitter();
    @Output() exportAction: EventEmitter<any> = new EventEmitter();

    @Input() filterField: any;
    @Input() filterValue: any;
    @Input() imagePrefix: string;

    @ViewChild('dt') datatable: DataTable;
    @ViewChild('gb') gb: any;
    @ViewChild('filterDepartment') filterDepartment: any;

    showSide: boolean = false;  // показать профиль
    profile_id: number;

    filterTimeout: any;
    firstFilter: boolean = true;

    constructor(private router: Router, private route: ActivatedRoute, private renderer: Renderer2) {
    }

    ngOnInit() {
        this.checkCatalog();
        this.route.paramMap.subscribe(
            params => {
                if (!this.slider) {
                    this.filterField = params.get('field');
                    this.filterValue = params.get('value');
                    if (!this.firstFilter) {
                        this.checkPreFilter();
                        this.datatable._filter();
                        if (this.showSide) {
                            this.toggleSide();
                        }
                    }
                }
            }
        );
    }

    getPhoto(photo) {
        if (!photo) {
            return null;
        }
        return this.imagePrefix + photo;
    }

    checkPreFilter() {
        if (this.route.snapshot.paramMap.get('field') && this.route.snapshot.paramMap.get('value') && this.firstFilter) {
            this.filterField = this.route.snapshot.paramMap.get('field');
            this.filterValue = this.route.snapshot.paramMap.get('value');
        }

        if (this.filterField && this.filterValue) {
            this.filter[this.filterField] = this.filterValue;
            if (this.filterValue === parseInt(this.filterValue, 10)) {
                this.datatable.filters[this.filterField] = {
                    value: parseInt(this.filterValue, 10),
                    matchMode: 'any'
                };
            } else {
                this.datatable.filters[this.filterField] = {
                    value: this.filterValue,
                    matchMode: 'any'
                };
            }
        }
    }

    ifDisabled(field) {
        return !!(this.filterField && this.filterValue && this.filterField === field);
    }

    ngAfterViewInit() {
        this.renderer.listen(
            this.filterDepartment.filterViewChild.nativeElement,
            'keyup',
            (event: KeyboardEvent) => {
                const key = event.which || event.keyCode;

                if (key === 13 || key === 40) {
                    this.filterDepartment.applyFocus();
                    /**
                     * из кода primeng/components/dropdown/dropdown.ts строка 419
                     * обработка нажатия клавиши вниз, чтобы после фильтрации выбрать первый отфильтрованнный элемент
                     */
                    const selectedItemIndex = this.filterDepartment.selectedOption ? this.filterDepartment.findOptionIndex(
                        this.filterDepartment.selectedOption.value,
                        this.filterDepartment.optionsToDisplay
                    ) : -1;
                    if (selectedItemIndex !== -1) {
                        const nextItemIndex = selectedItemIndex + 1;
                        if (nextItemIndex !== (this.filterDepartment.optionsToDisplay.length)) {
                            this.filterDepartment.selectedOption = this.filterDepartment.optionsToDisplay[nextItemIndex];
                            this.filterDepartment.selectedOptionUpdated = true;
                            this.filterDepartment.selectItem(event, this.filterDepartment.selectedOption);
                        }
                    } else if (this.filterDepartment.optionsToDisplay) {
                        this.filterDepartment.selectedOption = this.filterDepartment.optionsToDisplay[0];
                        this.filterDepartment.selectedOptionUpdated = true;
                        this.filterDepartment.selectItem(event, this.filterDepartment.selectedOption);
                    }
                }
            });
    }

    /**
     * Скрыть/показать профиль пользователя
     */
    toggleSide() {
        this.showSide = !this.showSide;
    }

    clearFilter() {
        if (this.filterField || this.filterValue) {
            this.filterField = null;
            this.filterValue = null;

            if (!this.slider) {
                this.router.navigate(['/search/student']);
            }
        }

        this.filter = new Filter();
        if (this.datatable.globalFilter && this.datatable.globalFilter.value) {
            this.datatable.globalFilter.value = '';
        }
        this.datatable.reset();
        this.datatable.sortField = 'fullname';
        this.datatable._filter();
    }

    /**
     * Скрыть / показать фильтры
     */
    toggleFilter() {
        this.filterShow = !this.filterShow;
    }

    /**
     * Применение глобального фильтра
     * @param gbt
     */
    globalFilterApply(gbt) {
        if (this.filterTimeout) {
            clearTimeout(this.filterTimeout);
        }
        if (gbt && gbt.value && gbt.value.length > 2) {
            this.filterTimeout = setTimeout(() => {
                this.datatable.globalFilter = gbt;
                this.datatable._filter();
                this.filterTimeout = null;
            }, this.filterDelay);
        }
        if (gbt && gbt.value === '') {
            this.datatable.globalFilter.value = '';
            this.datatable._filter();
        }
    }

    setProfileHeader() {
        if (this.profile && this.profile.role && this.profile.role.length > 0) {
            return this.profile.user.fullname;
        }
        return '';
    }

    /**
     * Установка и применение фильтра
     * @param name
     * @param value
     */
    setFilter(name, value) {
        switch (name) {
            case 'fullname':
                if (this.filterTimeout) {
                    clearTimeout(this.filterTimeout);
                }
                if (value && value.length > 2) {
                    this.filterTimeout = setTimeout(() => {
                        this.datatable.filter(value, name, 'any');
                        this.filterTimeout = null;
                    }, this.filterDelay);
                }
                if (value === '') {
                    this.datatable.filter(value, name, 'any');
                }
                break;
            default:
                this.datatable.filter(value, name, 'any');
        }

    }

    loadItemsLazy(event: any) {
        if (this.firstFilter) {
            this.checkPreFilter();
            this.firstFilter = false;
        }

        this.onFilter.emit(event);
    }

    /**
     * Устанавилвает фильтр из контекста (при клике на значение в таблице)
     * @param name string Название поля для фильтрации
     * @param value string Значение поля
     */
    setContextFilter(name, value) {
        this.filter = new Filter();
        this.datatable.filters = {};
        this.datatable.filteredValue = null;
        this.datatable.sortField = 'fullname';
        this.filter[name] = value;
        this.setFilter(name, value);
    }

    /**
     * заполнение каталогов если они пустые
     */
    checkCatalog() {
        if (!this.catalog.years || this.catalog.years.length === 0) {
            this.initCatalogYears(7);
        }
        if (!this.catalog.edu_forms || this.catalog.edu_forms.length === 0) {
            this.initCatalogForms();
        }
        if (!this.catalog.edu_courses || this.catalog.edu_courses.length === 0) {
            this.initCatalogCourses(6);
        }
    }

    initCatalogYears(number) {
        this.catalog.years = [];
        const year = new Date().getFullYear();
        this.catalog.years.push(
            {label: '(любой)', value: ''}
        );
        for (let i = 0, y = year; i < number; i++, y--) {
            this.catalog.years.push(
                {label: y, value: y}
            );
        }
    }

    initCatalogForms() {
        this.catalog.edu_forms = [];
        this.catalog.edu_forms.push(
            {label: '(любая)', value: ''},
            {label: 'очное обучение', value: 'full-time'},
            {label: 'заочное обучение', value: 'distance'},
            {label: 'очно-заочное обучение', value: 'part-time'},
        );
    }

    initCatalogCourses(number) {
        this.catalog.edu_courses = [];
        this.catalog.edu_courses.push(
            {label: '(любой)', value: ''}
        );
        for (let i = 1; i <= number; i++) {
            this.catalog.edu_courses.push(
                {label: i, value: i}
            );
        }
    }

    showProfile(item ?) {
        if (item) {
            this.profile_id = item.id;
        }

        if (this.profile_id > 0) {
            this.onViewProfile.emit(this.profile_id);
            this.toggleSide();
        } else {
            console.error('Ошибка: Не получен ID профиля');
        }
    }

    onDblRowclick() {
        this.showProfile();
    }

    onRowClick(event) {
        this.profile_id = event.data.id;
    }

    /**
     * Установка подсказки в фильтре подразделений
     * @param item
     * @returns {any}
     */
    getLabel(item) {
        if (item && item.tree && item.tree.length > 0) {
            let deps = '';
            for (let i = 0; i < item.tree.length; i++) {
                deps += '<div class="dep" style="padding-left: ' + (i * 8) + 'px">' + item.tree[i].title + '</div>';
            }
            return `<div class="tooltip-custom-content">` + deps + `</div>`;
        } else {
            return '';
        }

    }

    managerViewHandler(e) {
        this.onViewManager.emit({id: e.id});
    }

    export() {
        this.exportAction.emit({filter: this.datatable.createLazyLoadMetadata()});
    }

}

@NgModule({
    imports: [
        CommonModule,
        FormsModule,
        RouterModule,
        SharedModule,
        GlxSharedModule,
        DataTableModule,
        DropdownModule,
        TooltipModule,
        ProfileViewerModule,
        LoadingBannerModule,
        SliderModule
    ],
    exports: [StudentSearch],
    declarations: [StudentSearch]
})

export class StudentSearchModule {
}
